// Implementation of the CConversionProviderCommand
#include "stdafx.h"
#include "SimpleDataObject.h"
#include "DataObjectRowset.h"
#include "IGetAsOLEDBRowset_i.c"
#include "IDisconnectableRowset_i.c"
/////////////////////////////////////////////////////////////////////////////
// CConversionProviderCommand
HRESULT CConversionProviderCommand::Execute(
   IUnknown *pUnkOuter, 
   REFIID riid, 
   DBPARAMS *pParams, 
   LONG *pcRowsAffected, 
   IUnknown **ppRowset)
{
   ATLTRACE2(atlTraceDBProvider, 0, "CConversionProviderCommand::Execute()\n");

   // This code must be here as the parameters are connected to the command's accessor, not
   // the rowsets... and we dont copy the accessors across yet...

   if (m_strCommandText ==  _T("CONVERT"))
   {
      // We must have a paramter passed to us.

      if (!pParams || 
          !pParams->pData ||
          pParams->cParamSets != 1 || 
          pParams->hAccessor == 0)
      {
         ATLTRACE2(atlTraceDBProvider, 0, "CConversionProviderCommand::Execute called with no parameters\n");
         return E_FAIL;
      }
   
      DBACCESSORFLAGS dwAccessorFlags = 0;
      ULONG cBindings = 0; 
      DBBINDING *prgBindings = 0;

      HRESULT hr = GetBindings(pParams->hAccessor, 
            &dwAccessorFlags, 
            &cBindings, 
            &prgBindings );

      if (hr != S_OK)
      {
         return E_FAIL;
      }

      // can it be anything other than an unknown?

      VARIANT &param = *(VARIANT*)((unsigned char*)(pParams->pData) + prgBindings[0].obValue);

      if (param.vt != VT_UNKNOWN)
      {
         return E_FAIL;
      }

      IUnknown *pUnk = param.punkVal;

      // At this point we have a pointer to the object that we want to create a rowset on, 

      pUnk->AddRef();

      _IGetAsOLEDBRowset *pGetAsRowset = 0;

      hr = pUnk->QueryInterface(IID__IGetAsOLEDBRowset, (void**)&pGetAsRowset);

      pUnk->Release();

      if (SUCCEEDED(hr))
      {
         // we now have a pointer to the thing that knows how to give us a rowset...
         // Make it so...

         IUnknown *pOurUnknown = 0;
      
         hr = QueryInterface(IID_IUnknown, (void**)&pOurUnknown); 

         if (SUCCEEDED(hr))
         {
            hr = pGetAsRowset->GetAsRowset(pOurUnknown, pUnkOuter, riid, pcRowsAffected, ppRowset);

            pOurUnknown->Release();
         }

         if (SUCCEEDED(hr))
         {
            hr = (*ppRowset)->QueryInterface(IID_IDisconnectableRowset, (void**)&m_pRowset);
         }

         pGetAsRowset->Release();
      }

      return hr;
   }
   else
   {
      // Need to handle SQL inserts, updates and deletes here...

      ATLTRACE2(atlTraceDBProvider, 0, m_strCommandText);

      if (pcRowsAffected)
      {
         *pcRowsAffected = 1;
      }

      if (ppRowset)
      {
         *ppRowset = 0;
      }

      return S_OK;
   }
   
   return DB_E_NOTABLE;
}

HRESULT CConversionProviderCommand::Cancel()
{
   HRESULT hr = ICommandTextImpl<CConversionProviderCommand>::Cancel();

   ATLTRACE2(atlTraceDBProvider, 0, "CConversionProviderCommand::Cancel\n");

   if (SUCCEEDED(hr))
   {
      if (m_pRowset)
      {
         hr = m_pRowset->DisconnectRowset();
         m_pRowset->Release();
         m_pRowset = 0;
      }
   }

   return hr;
}
